// $Id: Distance.cc 216 2011-11-27 05:57:39Z peter $

// Copyright (C) 2004, 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Distance.h"

#include <cmath>
#include <vector>

#include <yat/utility/Alignment.h>
#include <yat/utility/Matrix.h>


namespace theplu {
namespace speclust {

Distance::Distance(void) {}

Distance::~Distance(void) {}

double Distance::score(const std::vector<double>& l1,
											 const std::vector<double>& l2,
											 double sigma) const
{
	std::vector<std::pair<size_t,size_t> > path;
	yat::utility::Matrix m;
	return score(l1,l2,sigma,m,path);
}

double Distance::score(const std::vector<double>& l1,
											 const std::vector<double>& l2,
											 double sigma,
											 yat::utility::Matrix& dot_matrix,
											 std::vector<std::pair<size_t,size_t> >& path) const
{
	dot_matrix	= yat::utility::Matrix(l1.size(),l2.size());
	for (size_t i=0; i<l1.size(); i++)
		for (size_t j=0; j<l2.size(); j++)
			dot_matrix(i,j) = match(l1[i],l2[j],sigma);

	return yat::utility::NeedlemanWunsch(dot_matrix, path, 0);
}


}} // end of namespace speclust and namespace theplu
