// $Id: Cluster.cc 271 2012-11-27 13:18:47Z peter $

// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér, Peter Johansson
// Copyright (C) 2011, 2012 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Cluster.h"
#include "pic.h"
#include "Peak.h"

#include <yat/statistics/Averager.h>

#include <algorithm>
#include <vector>

namespace theplu {
namespace speclust {

	Cluster::Cluster()
	  : min_(0), max_(0)
	{
	}


	Cluster::Cluster(const Cluster& s)
	  : min_(s.min()), max_(s.max()), averager_(s.averager()),
	    peaks_(s.peaks())
	{
	}


	void Cluster::add(const Peak* p)
	{
	  if (p->mass() < min_ || averager_.n()==0)
	    min_=p->mass();
	  if (p->mass() > max_ || averager_.n()==0)
	    max_=p->mass();

	  averager_.add(p->mass());
	  peaks_.push_back(p);
	}


	void Cluster::write_header(std::ostream& os,
														 const std::vector<Spectrum*>& spectra) const
	{
	  os << "average\t";
	  os << "std\t";
	  os << "N\t";
	  os << "min\t";
	  os << "max";
	  for (size_t i=0; i<spectra.size(); ++i)
	    os << "\t" << spectra[i]->name();
	  os << "\n";
	}

	void Cluster::write(std::ostream& os, const std::vector<Spectrum*>& spectra)
	{
	  os << averager().mean() << "\t";
	  os << averager().std() << "\t";
	  os << averager().n() << "\t";
	  os << min() << "\t";
	  os << max();
		// sort peaks with respect to the name of their
		// spectrum (alphabetical order). This assumes spectra
		// are also are sorted in alphabetical order
		std::sort(peaks_.begin(), peaks_.end(), SpectrumName());
		size_t peak_index=0;
		// loop over spectra
		for (size_t i=0; i<spectra.size(); ++i){
		  os << "\t";
			bool first = true;
		  while (peak_index<peaks_.size() &&
						 spectra[i]==peaks_[peak_index]->spectrum()) {
				if (!first)
					os << ";";
				else
					first = false;
			  os << peaks_[peak_index]->mass();
			  ++peak_index;
		  }
		}
		os << "\n";
	}

}} // end of namespace speclust and namespace thep
