// $Id: Parameter.cc 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Parameter.h"

#include <iostream>
#include <string>

#include <yat/utility/FileUtil.h>

#include <cstdlib>

namespace theplu {
namespace speclust {

	Parameter::Parameter(int argc, char *argv[])
		: cmd_(theplu::yat::utility::CommandLine()),
			help_(cmd_),
			peaksets_(cmd_, "peaksets", "file with peak sets",
								true, true, "r"),
			id_(cmd_, "id", "file with identities (as output from psm)",
					true, true, "r"),
			sigma_(cmd_, "sigma", "width [1.0]"),
			consensus_(cmd_, "consensus","output file for consensus result"),
			consensus_cutoff_(cmd_, "consensus-cutoff", "consensus cutoff [0]"),
			multiple_(cmd_, "multiple", "output file for consenus result"),
			multiple_cutoff_(cmd_, "multiple-cutoff", "multiple cutoff [0]"),
			pairwise_(cmd_, "pairwise", "output file for pairwise result"),
			pairwise_cutoff_(cmd_, "pairwise-cutoff", "consenus cutoff [0]")
	{
		sigma_.value(1.0);
		consensus_cutoff_.value(0);
		pairwise_cutoff_.value(0);
		multiple_cutoff_.value(0);
		consensus_.value("/dev/null");
		pairwise_.value("/dev/null");
		multiple_.value("/dev/null");
		std::ostringstream ss;
		ss << "Find peaks in common between different peaklists.\n\n"
			 << "In paiwise is each pair of spectra returned with any peak\n"
			 << "above cutoff (-pc).\n\n"
			 << "In consensus is each peak returned with its matches.\nOnly"
			 << " peaks with a sum of matches above cutoff (-cc) is returned.\n"
			 << "\n";
		help_.synopsis() = ss.str();

		try {
			cmd_.parse(argc, argv);
		}
		catch (theplu::yat::utility::cmd_error& e) {
			std::cerr << e.what() << std::endl;
			exit(EXIT_FAILURE);
		}
	}

}} // end of namespace speclust and namespace thep
