// $Id: Peak.cc 269 2012-11-25 03:45:41Z peter $

// Copyright (C) 2005, 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Peak.h"
#include "Spectrum.h"
#include <iostream>
#include <string>
#include <utility>

namespace theplu {
namespace speclust {

  Peak::Peak(const Spectrum* s, const double m)
    : mass_(m), spectrum_(s), cluster_(NULL)
  {
  }


  double Peak::score_sum(void) const
  {
    double s = 0.0;
    for (size_t i=0; i<matches().size(); i++)
      s+=matches()[i].second;
    return s;
  }


  bool Peak::assign(std::vector<Cluster>& c)
  {
    if (cluster_)
      return false;

    cluster_=&c.back();
    cluster_->add(this);
    for (size_t i=0; i<matches_.size(); i++)
      matches_[i].first->assign(c);

    return true;
  }


  bool PeakLess::operator()(const Peak* p1, const Peak* p2)
  {
    return p1->mass() < p2->mass();
  }

  std::ostream& operator<<( std::ostream& os, const Peak& p)
  {
    os << p.spectrum()->name();
    os << std::string("\t") << p.mass();
    // ugly fix to make table look nice!
    for (size_t i=1; i<p.matches().size(); i++) {
      os << "\t";
    }


    os << "\nPeak list:";
    for (size_t i=0; i<p.matches().size(); i++) {
      os << "\t" << p.matches()[i].first->spectrum()->name();
    }

    os << "\nMass:";
    for (size_t i=0; i<p.matches().size(); i++) {
      os << "\t" << p.matches()[i].first->mass();
    }

    os << "\nScore:";
    for (size_t i=0; i<p.matches().size(); i++) {
      os << "\t" << p.matches()[i].second;
    }

    os << "\nTotal score:\t" << p.score_sum();
    // ugly fix to make table look nice!
    for (size_t i=1; i<p.matches().size(); i++) {
      os << "\t";
    }

    return os;
  }
}} // end of namespace speclust and namespace thep
