#ifndef _theplu_speclust_bin_pic_peak_
#define _theplu_speclust_bin_pic_peak_

// $Id: Peak.h 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2005, 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Cluster.h"

#include <string>
#include <utility>
#include <vector>

// class for command peak
namespace theplu {
namespace speclust {

	class Spectrum;

	class Peak
	{
	public:
		//
		// Default constructor not implemented
		//
		Peak();

		//
		// Constructor taking spectrum name and peak mass.
		//
		Peak(const Spectrum*, const double);

		//
		// Copy Constructor. Not yet implemented.
		//
		Peak(const Peak&);

		inline void add(Peak* p, double d)
		{ matches_.push_back(std::make_pair(p,d)); }
		double score_sum(void) const;
		inline double mass(void) const { return mass_; }
		inline const std::vector<std::pair<Peak*, double> >& matches(void) const
		{ return matches_; }
		inline const Spectrum* spectrum(void) const { return spectrum_; }

		///
		/// assigning the peak, its matched peaks, and their
		/// matches et c. to the last Cluster in vector.
		///
		/// @return true if peak was not already assigned to a cluster
		///
		bool assign(std::vector<Cluster>&);

	private:
		const double mass_;
		const Spectrum* spectrum_;
		std::vector<std::pair<Peak*, double> > matches_; //score > pairwise cutoff
		Cluster* cluster_;

	}; // end of class Peak

	struct PeakLess
	{
		bool operator()(const Peak*, const Peak*);
	}; // end of class PeakLess

	std::ostream& operator<<( std::ostream&, const Peak& );

}} // end of namespace speclust and namespace theplu

#endif
