// $Id: Spectrum.cc 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Spectrum.h"
#include "DistancePearson.h"

namespace theplu {
namespace speclust {

	Spectrum::Spectrum()
	{
	}

	Spectrum::Spectrum(const std::string& s)
		: name_(s)
	{
	}

	void Spectrum::align(Spectrum* other, const double sigma, const double cutoff)
	{
		DistancePearson distance;
		yat::utility::Matrix dot_m;
		std::vector<std::pair<size_t,size_t> > path;
		distance.score(mass(), other->mass(), sigma, dot_m, path);
		
		// updating the realionship between matched peaks
		for (size_t i=0; i<path.size(); i++) {
			if (dot_m(path[i].first,path[i].second) > cutoff) {

				peaks()[path[i].first]->add(other->peaks()[path[i].second],
																 dot_m(path[i].first,path[i].second));
			
				other->peaks()[path[i].second]->add(peaks()[path[i].first],
																				 dot_m(path[i].first,path[i].second));
			}
		}

	}


}} // end of namespace speclust and namespace thep
