// $Id: Parameter.cc 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2004 Jari Häkkinen
// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Parameter.h"

#include <yat/utility/FileUtil.h>

#include <cstdlib>
#include <iostream>
#include <string>

Parameter::Parameter(const int argc,const char *argv[])
{
	using namespace std;
	// check if help requested
	for (int i=1; i<argc; i++)
		if (string(argv[i])==string("-help") || string(argv[i])==string("-h")) {
		  help();
		  exit(0);			// always exit after printing help
		}

	defaults();

	for (int i=1; i<argc; i++) {
		bool ok=false;
		string myargv(argv[i]);
		if (myargv==string("-fl"))
			if ((i+1)<argc) {
				filelist_file_=argv[++i];
				ok=true;
			}
		if (myargv==string("-mnp"))
			if ((i+1)<argc) {
				min_nof_peaks_=atoi(argv[++i]);
				ok=true;
			}
		if (myargv==string("-nfl"))
			if ((i+1)<argc) {
				new_filelist_file_=argv[++i];
				ok=true;
			}
		if (myargv==string("-ps"))
			if ((i+1)<argc) {
				peakset_file_=argv[++i];
				ok=true;
			}

		if (!ok)
			std::cerr << "# Parameter::Parameter     Invalid option: "
								<< argv[i] << '\n';
	}

	analyse();
}


void Parameter::analyse(void)
{
	using namespace theplu::yat::utility;
	bool ok=true;
	if (filelist_file().size() && FileUtil(filelist_file()).permissions("r")) {
		std::cerr << "Cannot access file list file " << filelist_file()
							<< std::endl;
		ok=false;
	}
	/*
	if (new_filelist_file().size() &&
			FileIO().access_rights(new_filelist_file(),"w")) {
		std::cerr << "Cannot create new file list file " << new_filelist_file()
							<< std::endl;
		ok=false;
	}
	if (peakset_file().size() && FileIO().access_rights(peakset_file(),"w")) {
		std::cerr << "Cannot create peak set file " << peakset_file()
							<< std::endl;
		ok=false;
	}
	*/

	if (!ok)
		std::exit(-1);
}



void Parameter::defaults(void)
{
	min_nof_peaks_=0;
	new_filelist_file_="out.list";
	peakset_file_="peakset.data";
}

void Parameter::help(void)
{
  std::cout << "Merge several (single) peak set files into one single\n";
  std::cout << "file with (several) peak sets. Filtering on minimum number\n";
  std::cout << "peaks in a set to be included can optionally be done.\n";
  std::cout << "\n";
  std::cout << "The program reads from stdin or an input file, and creates\n";
  std::cout << "output files.\n\n";
  std::cout << "available options are (default value in brackets[]):\n";
  std::cout << "  -fl: set the input (files list) filename\n";
  std::cout << "       File format defined by using 'find' as:\n";
  std::cout << "           find file_dir -name '*.peaks' > file.list\n";
  std::cout << "  -mnp: set the min acceptable number of peaks in a set [0]\n";
  std::cout << "        Peak set sizes below this limit are omitted.\n";
  std::cout << "  -nfl: set the output (files list) filename [out.list]\n";
  std::cout << "        File format defined by using 'find' as:\n";
  std::cout << "           find file_dir -name '*.peaks' > file.list\n";
  std::cout << "  -ps: set the output (peak sets) filename [peakset.data]\n";
  std::cout << "       File format is space separated masses, one line per\n";
  std::cout << "       spectrum.\n\n";
  std::cout << "Return: 0 - succesful termination\n";
  std::cout << "       65 - The input data was incorrect "
						<< "(only numbers allowed)\n";
  std::cout << "      255 - Undefined error" << std::endl;


}
