// $Id: psm.cc 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2004 Jari Häkkinen
// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér, Peter Johansson
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "psm.h"
#include "Parameter.h"

#include <yat/utility/utility.h>

#include <cstdlib>
#include <fstream>
#include <iostream>
#include <iterator>
#include <vector>

int main(const int argc,const char* argv[])
{
	using namespace std;

	Parameter option(argc,argv);

	vector<string> filename;
	if(option.filelist_file().size()) {
		ifstream is(option.filelist_file().c_str());
		is >> filename;
	}
	else
		cin >> filename;

	ofstream outpeaks(option.peakset_file().c_str());
	ofstream outlist(option.new_filelist_file().c_str());

	for (std::vector<std::string>::const_iterator i=filename.begin();
			 i!=filename.end(); i++) {
		ifstream in(i->c_str());
		if (!in.good()){
		  cerr << "psm: error: open " << *i << " file." << endl;
			return EXIT_FAILURE;
		}

		vector<string> peaks;
		while (!in.eof()) {
			string str;
			in >> str;
			if (!theplu::yat::utility::is<float>(str) && !str.empty()) {
			  cerr << "Error: '" << str
			       << "' is not a C-style number." << std::endl;
			  exit(65); // DATAERR exit
			}
			if (in.good())
				peaks.push_back(str);
		}
		if (peaks.size()>=option.min_nof_peaks()) {
				outpeaks << peaks << '\n';
				outlist << *i << '\n';
		}
		in.close();
	}
	outpeaks.close();
	outlist.close();
	return EXIT_SUCCESS;
}



std::istream& operator>>(std::istream& s,std::vector<std::string>& filename)
{
	while (!s.eof()) {
		std::string str;
		getline(s,str,'\n');
		if (s.good())
			filename.push_back(str);
	}
	return s;
}

std::ostream& operator<<(std::ostream& s,const std::vector<std::string>& v)
{
	for (std::vector<std::string>::const_iterator i=v.begin(); i!=v.end(); i++)
		s << *i << ' ';
	return s;
}
