// $Id: Parameter.cc 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2004 Markus Ringnér
// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "Parameter.h"

#include <yat/utility/FileUtil.h>

#include <cstdlib>
#include <iostream>
#include <string>

Parameter::Parameter(int argc, char *argv[])
{
	using namespace std;
	// check if help requested
	for (int i=1; i<argc; i++)
		if (string(argv[i])==string("-help") || string(argv[i])==string("-h")) {
			std::cout << "Generate a hierarchical tree from a distance matrix\n";
			std::cout << "\n";
			std::cout << "Four options are available:\n";
			std::cout << "  -dm: set the input distance matrix filename\n";
			std::cout << "       file format is a space separated symmetric matrix, "
								<< "one line per leaf\n";
			std::cout << "  -ln: set the input leaf names filename\n";
			std::cout << "       file format is one line per leaf name\n";
			std::cout << "  -lm: set the linkage method\n";
			std::cout << "       average, single or complete\n";
			std::cout << "  -tf: set the output tree file filename\n";
			std::cout << "       file format is postscript\n";
			exit(0);			// always exit after printing help
		}

	string linkage="average";
	tree_file_="tree.ps";

	for (int i=1; i<argc; i++) {
		bool ok=false;
		string myargv(argv[i]);
		if (myargv==string("-dm"))
			if ((i+1)<argc) {
				distance_file_=argv[++i];
				ok=true;
			}
		if (myargv==string("-ln"))
		  if ((i+1)<argc) {
		    leaf_file_=argv[++i];
		    ok=true;
		  }
		if (myargv==string("-lm"))
		  if ((i+1)<argc) {
		    linkage=argv[++i];
		    ok=true;
		  }

		if (myargv==string("-tf"))
		  if ((i+1)<argc) {
		    tree_file_=argv[++i];
		    ok=true;
		  }

		if (!ok)
			std::cerr << "# Parameter::Parameter     Invalid option: "
								<< argv[i] << '\n';
	}

	analyse(linkage);
}



void Parameter::analyse(const std::string& linkage)
{
	using namespace theplu::yat::utility;
	bool ok=true;
	if (linkage=="average")
	  linkage_='a';
	else if (linkage=="single")
	  linkage_='s';
	else if (linkage=="complete")
	  linkage_='m';
	else {
	  std::cerr << "Unknown linkage method: " << linkage << std::endl;
	  ok=false;
	}
	if(!distance_file().size()) {
	  std::cerr << "No distance file!" << std::endl;
	  ok = false;
	}
	if(!leaf_file().size()) {
	  std::cerr << "No leaf file!" << std::endl;
	  ok = false;
	}

	if (!ok)
	  std::exit(-1);
}
