// $Id: TreeClusterDistances.cc 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2004 Markus Ringnér
// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "TreeClusterDistances.h"
#include "TreeNode.h"

#include <yat/utility/Matrix.h>

// Standard
#include <iostream>
#include <fstream>

extern "C" {
#include "cluster.h"
}

namespace theplu {
namespace speclust {

  TreeClusterDistances::TreeClusterDistances(const yat::utility::Matrix& dist,
																						 char link)
  {
    cluster(dist,link);
  }

  void TreeClusterDistances::cluster(const yat::utility::Matrix& dist,
																		 char linkage)
  {
    nodes_.clear();

    if(dist.rows()!=dist.columns()) {
      std::cerr << "Error in TreeClusterDistances: "
								<< " not a symmetric distance matrix!\n";
      exit(-1);
    }

    // Copy distance matrix to double**
    double** distmatrix = new double*[dist.rows()];
    for (size_t i=0; i<dist.rows(); i++) {
      distmatrix[i] = new double[dist.rows()];
    }
    for (size_t i=0; i<dist.rows(); i++)
      for (size_t j=0; j<dist.rows(); j++)
				distmatrix[i][j]=dist(i,j);

    // Perform clustering
    const int nnodes = dist.rows() - 1;
		//    int node[nnodes][2];
		Node* nodes = treecluster (dist.rows(), 0, 0, 0, 0, 0, 0, linkage,
															 distmatrix);

		/*
		treecluster(dist.rows(), 0, 0, 0, 0, 0, 0, 0, linkage, node,
								distances, distmatrix);
		*/
		std::ofstream os("nodes.txt");
    os << "Node\tItem 1\tItem 2\tDistance\n";
    for(int i=0; i<nnodes; i++) {
      os << -i-1 << '\t' << nodes[i].left << '\t' <<  nodes[i].right << '\t' <<
				nodes[i].distance << '\n';
      TreeNode tn(nodes[i].right, nodes[i].left, nodes[i].distance);
      nodes_.push_back(tn);
    }
		os.close();
		free(nodes);

    // deallocate distmatrix
    for (size_t i=0; i<dist.rows(); i++) {
      delete[] distmatrix[i];
    }
    delete[] distmatrix;

  }
}} // end of namespaces
