// $Id: tcd.cc 228 2012-09-13 22:45:17Z peter $

// Copyright (C) 2004 Markus Ringnér
// Copyright (C) 2005 Peter Johansson
// Copyright (C) 2006 Markus Ringnér
// Copyright (C) 2011 Peter Johansson
//
// This file is part of speclust, http://bioinfo.thep.lu.se/speclust.html
//
// Speclust is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 3 of the License, or (at your option) any later
// version.
//
// Speclust is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// speclust. If not, see <http://www.gnu.org/licenses/>.

#include "DrawTree.h"
#include "Parameter.h"
#include "TreeClusterDistances.h"
#include "TreeNode.h"

// yat
#include <yat/utility/Matrix.h>

// Standard includes
////////////////////
#include <cassert>
#include <fstream>
#include <iostream>
#include <vector>
#include <string>

// forward stuff
std::istream& operator>>(std::istream&,std::vector<std::string>&);

int main(int argc, char* argv[])
{
  using namespace std;
  using namespace theplu::speclust;

  Parameter option(argc,argv);
  ifstream is;
  is.open(option.distance_file().c_str());
  theplu::yat::utility::Matrix distances(is);
  is.close();
	assert(distances.rows()==distances.columns());

  is.open(option.leaf_file().c_str());
  vector<string> leaves;
  string leaf;
  while(std::getline(is,leaf)) {
    leaves.push_back(leaf);
	}
  is.close();
	assert(distances.rows()==leaves.size());

  TreeClusterDistances tcg(distances,option.linkage());

  DrawTree drawtree(tcg.nodes(),leaves);
  drawtree.make_picture(option.tree_file());

  return 0;
}


std::istream& operator>>(std::istream& s,
												 std::vector<std::string>& st)
{
	while (!s.eof()) {
		std::string row;
		s >> row;
		if (!s.fail())
			st.push_back(row);
	}
	return s;
}
